@@ -192,6 +192,7 @@ class Logic(BaseLogic):
         self.backward_outputs = options.pop('backward_outputs', None)
         self._grad_scaler = options.pop('grad_scaler', None)
         self._autocast = options.pop('autocast', False)
+        self._backward_fn = options.pop('backward_function', None)
 
         if not _amp_enabled:
             if self._grad_scaler is not None or self._autocast:
@@ -214,10 +215,9 @@ class Logic(BaseLogic):
         return model(batch)
 
     def _backward(self, outputs: Dict[str, Any]) -> None:
-        target = _normalize_outputs(outputs)
         to_backward = set()
         if self.backward_outputs is None:
-            for _, v in target.items():
+            for _, v in outputs.items():
                 if isinstance(v, torch.Tensor) and v.grad_fn is not None and (
                     (
                         v.numel() == 1
@@ -233,15 +233,18 @@ class Logic(BaseLogic):
                 backward_outputs = (backward_outputs,)
             for k in backward_outputs:
                 try:
-                    to_backward.add(target[k])
+                    to_backward.add(outputs[k])
                 except KeyError:
                     warnings.warn(
                         'Couldn\'t find requested backward value: '
-                        f'{k} in {target.keys()}'
+                        f'{k} in {outputs.keys()}'
                     )
 
         for v in to_backward:
-            v.backward()  # type: ignore[no-untyped-call]
+            if self._backward_fn is None:
+                v.backward()  # type: ignore[no-untyped-call]
+            else:
+                self._backward_fn(v)
 
     def train_epoch_begin(
             self,
@@ -288,9 +291,8 @@ class Logic(BaseLogic):
         with torch_autocast(enabled=self._autocast):
             optimizers[self.model_name].zero_grad()
             outs = self._forward(models[self.model_name], batch)
-            to_back_outs = outs
+            to_back_outs = _normalize_outputs(outs)
             if self._grad_scaler is not None:
-                to_back_outs = _normalize_outputs(outs)
                 assert (
                     len(to_back_outs) == 1
                 ), "loss scaling with multiple outputs is not supported"
