@@ -1,5 +1,5 @@
 from pytorch_pfn_extras.profiler._record import record  # NOQA
 from pytorch_pfn_extras.profiler._record import record_function  # NOQA
 from pytorch_pfn_extras.profiler._record import record_iterable  # NOQA
-from pytorch_pfn_extras.profiler._time_summary import time_summary  # NOQA
+from pytorch_pfn_extras.profiler._time_summary import get_time_summary  # NOQA
 from pytorch_pfn_extras.profiler._time_summary import TimeSummary  # NOQA
