@@ -4,7 +4,7 @@ import copy
 from pytorch_pfn_extras.profiler import record
 import time
 from typing import (
-    Any, Dict, Iterable, Generator, List, Mapping, Optional, Union, TYPE_CHECKING
+    Any, Dict, Iterable, Generator, Mapping, Optional, Sequence, Union, TYPE_CHECKING
 )
 import warnings
 
@@ -55,7 +55,7 @@ class _ManagerProxy:
         return self._manager._iters_per_epoch
 
     @property
-    def models(self) -> Dict[str, torch.nn.Module]:
+    def models(self) -> Mapping[str, torch.nn.Module]:
         return self._manager.models
 
     @property
@@ -113,7 +113,7 @@ class _ManagerExecutionProxy(_ManagerProxy):
         return self._manager.execution
 
     @property
-    def models(self) -> Dict[str, torch.nn.Module]:
+    def models(self) -> Mapping[str, torch.nn.Module]:
         raise RuntimeError('Models are not available during execution phase.')
 
     @property
@@ -136,7 +136,7 @@ class _BaseExtensionsManager:
             optimizers: Union[torch.optim.Optimizer,
                               Mapping[str, torch.optim.Optimizer]],
             max_epochs: int,
-            extensions: Optional[List['extension_module.ExtensionLike']],
+            extensions: Optional[Sequence['extension_module.ExtensionLike']],
             out_dir: str,
             writer: Optional[writing.Writer],
             stop_trigger: 'trigger_module.TriggerLike' = None,
@@ -160,6 +160,7 @@ class _BaseExtensionsManager:
         self.reporter = reporting.Reporter()
         self._transform_model = transform_model
         self._start_extensions_called = False
+        self._run_on_error_called = False
 
         # Indicates whether models can be accessed from extensions in the
         # current iteration.
@@ -222,7 +223,7 @@ class _BaseExtensionsManager:
             '`needs_model_state = True` attribute.')
 
     @property
-    def models(self) -> Dict[str, torch.nn.Module]:
+    def models(self) -> Mapping[str, torch.nn.Module]:
         self.start_extensions()
         self._check_model_available()
         models = {k: self._transform_model(k, v)
@@ -435,6 +436,14 @@ class _BaseExtensionsManager:
         else:
             raise ValueError('extension %s not found' % name)
 
+    def _run_on_error(self, exc: Exception) -> None:
+        if not self._run_on_error_called:
+            self._run_on_error_called = True
+            tb = exc.__traceback__
+            assert tb is not None
+            for _, entry in self.extensions:
+                entry.extension.on_error(self, exc, tb)
+
     def run_extensions(
             self, *, completed: bool = True, only_iterations: bool = True) -> None:
         if completed:
@@ -595,7 +604,7 @@ class ExtensionsManager(_BaseExtensionsManager):
             max_epochs: int,
             *,
             iters_per_epoch: int,
-            extensions: Optional[List['extension_module.ExtensionLike']] = None,
+            extensions: Optional[Sequence['extension_module.ExtensionLike']] = None,
             out_dir: str = 'result',
             stop_trigger: 'trigger_module.TriggerLike' = None,
             writer: Optional[writing.Writer] = None,
@@ -640,15 +649,17 @@ class ExtensionsManager(_BaseExtensionsManager):
                 yield
                 for name in step_optimizers_names:
                     self._optimizers[name].step()
-            finally:
                 self.iteration += 1
                 self.run_extensions(completed=True, only_iterations=True)
+            except Exception as e:
+                self._run_on_error(e)
+                raise
 
     @contextlib.contextmanager
     def run_iteration(
             self,
             *,
-            step_optimizers: Optional[List[str]] = None
+            step_optimizers: Optional[Sequence[str]] = None
     ) -> Generator[IterationNotification, None, None]:
         """Context manager to run an iteration.
 
@@ -664,7 +675,7 @@ class ExtensionsManager(_BaseExtensionsManager):
             self.start_extensions()
 
         notification = IterationNotification()
-        step_optimizers_names = []
+        step_optimizers_names: Sequence[str] = []
         if step_optimizers is not None:
             step_optimizers_names = step_optimizers
         self.observation = {}
@@ -676,7 +687,6 @@ class ExtensionsManager(_BaseExtensionsManager):
                 if notification._is_completed:
                     for name in step_optimizers_names:
                         self._optimizers[name].step()
-            finally:
                 # The iteration count is increased just before calling the
                 # extensions.
                 if notification._is_completed:
@@ -691,6 +701,9 @@ class ExtensionsManager(_BaseExtensionsManager):
                     # needs to run regardless of the training state
                     self.execution += 1
                     self.run_extensions(completed=False, only_iterations=False)
+            except Exception as e:
+                self._run_on_error(e)
+                raise
 
         if self._internal_stop_trigger(self):
             self._finalize_extensions()
@@ -724,7 +737,7 @@ class IgniteExtensionsManager(_BaseExtensionsManager):
                               Mapping[str, torch.optim.Optimizer]],
             max_epochs: int,
             *,
-            extensions: Optional[List['extension_module.ExtensionLike']] = None,
+            extensions: Optional[Sequence['extension_module.ExtensionLike']] = None,
             out_dir: str = 'result',
             writer: Optional[writing.Writer] = None
     ) -> None:
