@@ -24,9 +24,11 @@ _intermediate_prefix = "intermedaite:"
 
 
 class _ComparableHandler(_handler_module.BaseHandler):
-    def __init__(self, handler, name, save_outs_cb, trigger=None, *, dir=None):
+    def __init__(
+            self, handler, name, get_target_cb, compare_cb, trigger=None, *, dir=None):
         self._handler = handler
-        self._save_outs_cb = save_outs_cb
+        self._get_target_cb = get_target_cb
+        self._compare_cb = compare_cb
         self.name = name
         self._trigger = trigger
         self._intermediate_values = {}
@@ -58,9 +60,8 @@ class _ComparableHandler(_handler_module.BaseHandler):
         self._handler.train_validation_end(trainer, evaluator)
 
     def train_step(self, trainer, batch_idx, batch, complete_fn):
-        self._intermediate_values[batch_idx] = {}
-        self._intermediate_counts[batch_idx] = {}
         self._batch_idx = batch_idx
+        self._reset_intermediate_values()
         self._handler.train_step(trainer, batch_idx, batch, complete_fn)
 
     def train_post_step(self, trainer, batch_idx, batch, outputs):
@@ -74,7 +75,7 @@ class _ComparableHandler(_handler_module.BaseHandler):
         manager = _ManagerProxy(trainer.manager)
         self._handler.train_post_step(trainer, batch_idx, batch, outputs)
         if self._trigger is None or self._trigger(manager):
-            self._save_outs_cb(self, trainer, batch_idx, outputs)
+            self._compare(trainer, batch_idx, outputs)
 
     def eval_setup(self, evaluator, loader):
         return self._handler.eval_setup(evaluator, loader)
@@ -84,9 +85,8 @@ class _ComparableHandler(_handler_module.BaseHandler):
         _thread_local.handler = self
 
     def eval_step(self, evaluator, batch_idx, batch, complete_fn):
-        self._intermediate_values[batch_idx] = {}
-        self._intermediate_counts[batch_idx] = {}
         self._batch_idx = batch_idx
+        self._reset_intermediate_values()
         self._handler.eval_step(evaluator, batch_idx, batch, complete_fn)
 
     def eval_loop_end(self, evaluator):
@@ -94,7 +94,16 @@ class _ComparableHandler(_handler_module.BaseHandler):
 
     def eval_post_step(self, evaluator, batch_idx, batch, outputs):
         self._handler.eval_post_step(evaluator, batch_idx, batch, outputs)
-        self._save_outs_cb(self, evaluator, batch_idx, outputs)
+        self._compare(evaluator, batch_idx, outputs)
+
+    def _compare(self, engine, batch_idx, outputs):
+        outputs = _logic._normalize_outputs(outputs)
+        target = self._get_target_cb(self, engine, batch_idx, outputs)
+        self._compare_cb(self, engine, batch_idx, target)
+
+    def _reset_intermediate_values(self) -> None:
+        self._intermediate_values[self._batch_idx] = {}
+        self._intermediate_counts[self._batch_idx] = {}
 
     def _add_intermediate_value(self, name: str, value: torch.Tensor) -> None:
         if self._intermediate_values is None:
@@ -113,7 +122,15 @@ class _ComparableHandler(_handler_module.BaseHandler):
         values[name] = value
 
 
-def get_default_comparer(rtol=1e-04, atol=0, equal_nan=True, msg=None):
+def _overwrite_handler(engine, *args, **kwargs) -> None:
+    engine.handler = _ComparableHandler(engine.handler, *args, **kwargs)
+    evaluator = getattr(engine, 'evaluator', None)
+    if evaluator is not None:
+        # For trainer with evaluator
+        evaluator.handler = _ComparableHandler(evaluator.handler, *args, **kwargs)
+
+
+def get_default_comparer(rtol=1e-04, atol=0, equal_nan=True):
     """Creates default comparer function.
 
     The created function will compare the outputs by using
@@ -123,22 +140,16 @@ def get_default_comparer(rtol=1e-04, atol=0, equal_nan=True, msg=None):
         rtol (float): Relative tolerance.
         atol (float): Absolute tolerance.
         equal_nan (bool): If ``True``, NaNs will be ignored.
-        msg (str): Error message to be printed in case of failure.
     """
-    def compare_fn(backend1, backend2, name, val1, val2):
-        try:
-            torch.testing.assert_allclose(
-                # TODO select the device where
-                # the tensors will be compared?
-                val1.cpu().detach(),
-                val2.cpu().detach(),
-                rtol=rtol,
-                atol=atol,
-                equal_nan=equal_nan,
-            )
-        except AssertionError as e:
-            err_msg = msg or f"Comparing '{backend1}' and '{backend2}' in '{name}'"
-            raise AssertionError(err_msg + '\n' + str(e))
+    def compare_fn(backend1: str, backend2: str, name: str, val1: Any, val2: Any):
+        # TODO select the device where
+        # the tensors will be compared?
+        if isinstance(val1, torch.Tensor):
+            val1 = val1.cpu().detach()
+        if isinstance(val2, torch.Tensor):
+            val2 = val2.cpu().detach()
+        torch.testing.assert_allclose(
+            val1, val2, rtol=rtol, atol=atol, equal_nan=equal_nan)
 
     return compare_fn
 
@@ -146,6 +157,27 @@ def get_default_comparer(rtol=1e-04, atol=0, equal_nan=True, msg=None):
 _default_comparer = get_default_comparer()
 
 
+def _compare_targets(compare_fn, targets, baseline, batch_idx):
+    names = list(targets.keys())
+    if baseline is None:
+        baseline = names[0]
+    keys = sorted(targets[baseline].keys())
+
+    err_msg = ''
+    for backend in set(names) - set([baseline]):
+        for val_name in keys:
+            out1 = targets[baseline][val_name]
+            out2 = targets[backend][val_name]
+            try:
+                compare_fn(baseline, backend, val_name, out1, out2)
+            except AssertionError as e:
+                err_msg += (
+                    f"Comparing '{baseline}' and '{backend}' in '{val_name}'\n"
+                    f"{str(e)}\n")
+    if err_msg:
+        raise AssertionError(f'Batch: {batch_idx}\n' + str(err_msg))
+
+
 class _ComparerBase:
     def __init__(
             self, engines, *,
@@ -173,15 +205,7 @@ class _ComparerBase:
         self._iters = {}
         # engines must be a dict
         for name, engine in engines.items():
-            engine.handler = _ComparableHandler(
-                engine.handler, name, self.compare_targets
-            )
-            child_evaluator = getattr(engine, 'evaluator', None)
-            if child_evaluator is not None:
-                # For trainer with evaluator
-                child_evaluator.handler = _ComparableHandler(
-                    child_evaluator.handler, name, self.compare_targets)
-                # child_evaluator.handler = engine.handler
+            _overwrite_handler(engine, name, self._get_target, self.compare_targets)
 
     def _assert_incompatible_trigger(self, condition):
         if not condition:
@@ -231,18 +255,18 @@ class _ComparerBase:
             for future in concurrent.futures.as_completed(futures):
                 future.result()
 
-    def _add_target(self, handle, models, outputs):
-        raise NotImplementedError('Comparers must override _add_target')
+    def _get_target(self, handle, models, outputs):
+        raise NotImplementedError('Comparers must override _get_target')
 
-    def compare_targets(self, handle, engine, batch_idx, outputs):
-        self._iters[handle.name] += 1
-        if (self.n_iters is None) or (self._iters[handle.name] % self.n_iters == 0):
+    def compare_targets(self, handler, engine, batch_idx, target):
+        self._iters[handler.name] += 1
+        if (self.n_iters is None) or (self._iters[handler.name] % self.n_iters == 0):
             # Save the outputs of this iteration
             with self.report_lock:
-                self._add_target(handle, engine.models, outputs)
+                self.targets[handler.name] = target
                 if len(self.targets.keys()) == len(self.engines.keys()):
                     # all outputs have been filled, lets compare and reset
-                    self._compare_targets()
+                    _compare_targets(self.compare_fn, self.targets, None, batch_idx)
                     self.targets = {}
                 self._assert_incompatible_trigger(not self._finalized)
             # Excplicitly synchronize
@@ -250,17 +274,6 @@ class _ComparerBase:
             self.barrier.wait()
             self._semaphore.acquire()
 
-    def _compare_targets(self):
-        names = list(self.targets.keys())
-        for i, name in enumerate(names):
-            for target in self.targets[name]:
-                for j in range(i + 1, len(names)):
-                    to_compare = names[j]
-                    target_1 = self.targets[name][target]
-                    target_2 = self.targets[to_compare][target]
-                    self.compare_fn(
-                        name, to_compare, target, target_1, target_2)
-
 
 class OutputsComparer(_ComparerBase):
     def __init__(
@@ -296,14 +309,13 @@ class OutputsComparer(_ComparerBase):
         super().__init__(engines, compare_fn=compare_fn, concurrency=concurrency)
         self.to_compare_keys = to_compare_keys
 
-    def _add_target(self, handle, models, outputs):
-        outputs = _logic._normalize_outputs(outputs)
+    def _get_target(self, handle, engine, batch_idx, outputs):
         keys = (
             self.to_compare_keys
             if self.to_compare_keys is not None
             else outputs.keys()
         )
-        self.targets[handle.name] = {key: outputs[key] for key in keys}
+        return {key: outputs[key] for key in keys}
 
 
 class ModelComparer(_ComparerBase):
@@ -356,13 +368,11 @@ class ModelComparer(_ComparerBase):
                     raise ValueError(
                         f'didnt find a match for {tc_k} in the model')
 
-    def _add_target(self, handle, models, outputs):
-        outputs = _logic._normalize_outputs(outputs)
-        sdict = models['main'].state_dict()
+    def _get_target(self, handle, engine, batch_idx, outputs):
+        sdict = engine.models['main'].state_dict()
         if self._preprocessed_keys is None:
             self._preprocess_keys(sdict)
-        self.targets[handle.name] = {
-            key: sdict[key] for key in self._preprocessed_keys}
+        return {key: sdict[key] for key in self._preprocessed_keys}
 
 
 # New comparer interface
@@ -405,6 +415,7 @@ class Comparer:
             concurrency=None,
             outputs=True,
             params=False,
+            baseline=None,
     ):
         """A class for comparison of iteration outputs and model parameters.
 
@@ -422,6 +433,8 @@ class Comparer:
                 A set of keys of output dict to compare.
             params (tuple of str or bool):
                 A set of keys of model parameters to compare.
+            baseline (str, optional):
+                The baseline engine that is assumed to be correct.
 
         Examples:
             >>> trainer_cpu = ppe.engine.create_trainer(
@@ -439,11 +452,12 @@ class Comparer:
         self._targets = {}
         self._output_keys = outputs
         self._param_keys = params
+        self._baseline = baseline
         self._finalized = False
         self._concurrency = concurrency  # Upper limit of semaphore size
         self._semaphore = None  # Sempaphore for training step execution
         self._barrier = None  # Synchronizes iteration timing
-        self._report_lock = threading.Lock()  # Locks `Comparer._add_target`
+        self._report_lock = threading.Lock()  # Locks `Comparer._get_target`
 
         if trigger is None:
             self._trigger = trigger_module.get_trigger((1, "epoch"))
@@ -451,17 +465,15 @@ class Comparer:
             self._engine_type = _trainer.Trainer
             self._trigger = trigger_module.get_trigger(trigger)
 
-    def _add_target(self, handler, models, outputs, batch_idx):
-        outputs = _logic._normalize_outputs(outputs)
+    def _get_target(self, handler, engine, batch_idx, outputs):
         targets = {}
-
         outputs = _filter(self._output_keys, lambda: outputs)
         targets.update({
             k if k.startswith(_intermediate_prefix) else 'output:' + k: v
             for k, v in outputs.items()})
         targets.update(handler._intermediate_values.pop(batch_idx))
 
-        params = _filter(self._param_keys, models['main'].state_dict)
+        params = _filter(self._param_keys, engine.models['main'].state_dict)
         targets.update({'param:' + k: v for k, v in params.items()})
         return targets
 
@@ -469,21 +481,6 @@ class Comparer:
         if not condition:
             raise ValueError("Engines have different triggers.")
 
-    def _compare_outputs(self):
-        names = list(self._engines.keys())
-        backend1 = names[0]
-        keys1 = sorted(self._targets[backend1].keys())
-        for backend2 in names[1:]:
-            keys2 = sorted(self._targets[backend2].keys())
-            if keys1 != keys2:
-                raise ValueError(
-                    'Reported variable names incompatible\n'
-                    f'{backend1}: {keys1}\n{backend2}: {keys2}')
-            for val_name in keys1:
-                out1 = self._targets[backend1][val_name]
-                out2 = self._targets[backend2][val_name]
-                self._compare_fn(backend1, backend2, val_name, out1, out2)
-
     @staticmethod
     def _get_filename(engine, batch_idx):
         name = type(engine).__name__
@@ -493,14 +490,14 @@ class Comparer:
         name += f'_iter_{batch_idx}'
         return name
 
-    def _compare_targets(self, handler, engine, batch_idx, outputs):
+    def _compare_targets(self, handler, engine, batch_idx, target):
         # Save the outputs of this iteration
         with self._report_lock:
-            self._targets[handler.name] = self._add_target(
-                handler, engine.models, outputs, batch_idx)
+            self._targets[handler.name] = target
             if len(self._targets.keys()) == len(self._engines.keys()):
                 # all outputs have been filled, lets compare and reset
-                self._compare_outputs()
+                _compare_targets(
+                    self._compare_fn, self._targets, self._baseline, batch_idx)
                 self._targets = {}
             self._assert_incompatible_trigger(not self._finalized)
 
@@ -535,40 +532,24 @@ class Comparer:
         if name in self._engines.keys():
             raise ValueError(f"Engine named {name} already registered")
 
-        engine.handler = _ComparableHandler(
-            engine.handler, name, self._compare_targets, self._trigger)
-
-        child_evaluator = getattr(engine, 'evaluator', None)
-        if child_evaluator is not None:
-            # For trainer with evaluator
-            child_evaluator.handler = _ComparableHandler(
-                child_evaluator.handler, name, self._compare_targets, self._trigger)
+        _overwrite_handler(
+            engine, name, self._get_target, self._compare_targets, self._trigger)
 
         self._engines[name] = engine, args, kwargs
 
-    def _load_dump(self, engine, batch_idx):
-        dir = engine.handler._dir
+    def _load_dump(self, handler, engine, batch_idx, outputs):
         name = self._get_filename(engine, batch_idx)
-        return torch.load(f'{dir}/{name}')
-
-    def _compare_dump(self, handler, engine, batch_idx, outputs):
-        # Save the outputs of this iteration
-        with self._report_lock:
-            self._targets[handler.name] = self._load_dump(engine, batch_idx)
-            if len(self._targets.keys()) == len(self._engines.keys()):
-                # all outputs have been filled, lets compare and reset
-                self._compare_outputs()
-                self._targets = {}
-            self._assert_incompatible_trigger(not self._finalized)
-
-        # Excplicitly synchronize
-        self._semaphore.release()
-        try:
-            self._barrier.wait()
-        finally:
-            self._semaphore.acquire()
+        return torch.load(f'{engine.handler._dir}/{name}')
 
     def add_dump(self, name, dir):
+        """Add an engine to compare variables.
+
+        Args:
+            name (str):
+                The name of dump.
+            dir (str):
+                The directory that the results are saved to.
+        """
         with open(f'{dir}/summary') as f:
             summary = json.loads(f.read())
 
@@ -583,6 +564,7 @@ class Comparer:
         model = DummyModel()
         ppe.to(model, 'cpu')
 
+        # Create a dummy engine
         engine = None
         args = []
         if summary['evaluator']:
@@ -597,32 +579,29 @@ class Comparer:
             )
             args = [[None] * summary['train_len']] + args
 
-        engine.handler = _ComparableHandler(
-            engine.handler, name, self._compare_dump, self._trigger, dir=dir)
-
-        child_evaluator = getattr(engine, 'evaluator', None)
-        if child_evaluator is not None:
-            # For trainer with evaluator
-            child_evaluator.handler = engine.handler
+        _overwrite_handler(
+            engine, name, self._load_dump, self._compare_targets,
+            self._trigger, dir=dir)
 
         self._engines[name] = engine, args, {}
 
-    def _dump_targets(self, dir):
-        def dump_targets(handler, engine, batch_idx, outputs):
-            outputs = _logic._normalize_outputs(outputs)
-            targets = self._add_target(handler, engine.models, outputs, batch_idx)
-            name = self._get_filename(engine, batch_idx)
-            torch.save(targets, f'{dir}/{name}')
-        return dump_targets
+    def _dump_targets(self, handler, engine, batch_idx, target):
+        name = self._get_filename(engine, batch_idx)
+        torch.save(target, f'{engine.handler._dir}/{name}')
 
     def dump(self, engine, dir, *args, **kwargs):
-        engine.handler = _ComparableHandler(
-            engine.handler, None, self._dump_targets(dir), self._trigger)
+        """Add an engine to compare variables.
 
-        child_evaluator = getattr(engine, 'evaluator', None)
-        if child_evaluator is not None:
-            # For trainer with evaluator
-            child_evaluator.handler = engine.handler
+        Args:
+            engine (Trainer or Evaluator):
+                An engine to compare variables.
+            dir (str):
+                Name of the directory that the results are saved to.
+            *args and **kwargs:
+                Arguments passed to ``engine.run``.
+        """
+        _overwrite_handler(
+            engine, None, self._get_target, self._dump_targets, self._trigger, dir=dir)
 
         engine.run(*args, **kwargs)
         engine.handler = engine.handler._handler
